/* extension.js
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

import GObject from 'gi://GObject';
import St from 'gi://St';
import Clutter from "gi://Clutter";
import GLib from "gi://GLib";
import Gio from "gi://Gio";

import {Extension, gettext as _} from 'resource:///org/gnome/shell/extensions/extension.js';
import * as PanelMenu from 'resource:///org/gnome/shell/ui/panelMenu.js';
import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';
import * as AppFavorites from "resource:///org/gnome/shell/ui/appFavorites.js";

import * as Main from 'resource:///org/gnome/shell/ui/main.js';

const Indicator = GObject.registerClass(
class Indicator extends PanelMenu.Button {
    _init() {
        super._init(0.5, _('Panel Dash'));
        this.name = 'panelDash';
        
        this.add_child(new St.Label({
            text: _('Dash'),
            y_align: Clutter.ActorAlign.CENTER,
        }));
        
        this.build_menu();

	    this._settings_id =
                global.settings.connect("changed::" + AppFavorites.getAppFavorites().FAVORITE_APPS_KEY,
					() => {
					    this.menu.removeAll();					    
					    this.build_menu();
					});     
    }
    
    add_item(app) {
        let item = new PopupMenu.PopupBaseMenuItem;
        this.menu.addMenuItem(item);
        let box = new St.BoxLayout({vertical: false,
				                    pack_start: false,
				                    style_class: "dash-menu-box"});
        item.actor.add_child(box);
        let icon = app.create_icon_texture(24);
        box.add_child(icon);
        let label = new St.Label({text: app.get_name(),
                                  y_align: Clutter.ActorAlign.CENTER,});
        box.add_child(label);
        item.connect("activate", () => {
            app.open_new_window(-1);
            Main.overview.hide();
            });
    }

    add_item2() {
        let item = new PopupMenu.PopupBaseMenuItem;
        this.menu.addMenuItem(item);
        let box = new St.BoxLayout({vertical: false,
				    pack_start: false,
				    style_class: "dash-menu-box"});
        item.actor.add_child(box);
        let icon = new St.Icon({icon_name: 'view-app-grid',
                                icon_size: 24});
        box.add_child(icon);
        let label = new St.Label({
                            text: _('Apps'),
                            y_align: Clutter.ActorAlign.CENTER,});
        box.add_child(label);
        item.connect("activate", () => {
            if(Main.overview.dash.showAppsButton.checked) {
            Main.overview.dash.showAppsButton.checked = false;
            }
            else {
            if (Main.overview.shouldToggleByCornerOrButton())
                Main.overview.show();
            Main.overview.dash.showAppsButton.checked = true;
            }
        });
    }

    build_menu() {
        let separator = new PopupMenu.PopupSeparatorMenuItem('');
        //this.menu.addMenuItem(separator);
        AppFavorites.getAppFavorites().getFavorites().forEach((app) => {
            this.add_item(app);
        });
        this.add_item2();
    }
    
    destroy() {
        if (this._settings_id) {
            global.settings.disconnect(this._settings_id);
        }
        super.destroy();
    }
    
    
});

export default class DashtoPanelMenuIndicatorExtension extends Extension {
    enable() {
        this._indicator = new Indicator();
        const index = Main.sessionMode.panel.left.indexOf('activities') + 1;
        Main.panel.addToStatusArea(this.uuid, this._indicator, index, 'left');
    }

    disable() {
        this._indicator.destroy();
        this._indicator = null;
        
    }
}
